<?php

namespace Drupal\helpdesk_integration\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Renderer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the helpdesk issue entity edit forms.
 */
class Issue extends ContentEntityForm {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected Renderer $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): Issue {
    /** @var \Drupal\helpdesk_integration\Form\Issue $instance */
    $instance = parent::create($container);
    $instance->setRenderer($container->get('renderer'));
    return $instance;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Exception
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $entity = $this->getEntity();
    $result = $entity->save();
    $link = $entity->toLink($this->t('View'))->toRenderable();

    $message_arguments = [
      '%label' => $this->entity->label(),
    ];
    $logger_arguments = $message_arguments + [
      'link' => $this->renderer->render($link),
    ];

    $this->messenger()->addStatus(
      $this->t('New helpdesk issue %label has been created.',
        $message_arguments));
    $this->logger('helpdesk_integration')
      ->notice('Created new helpdesk issue %label', $logger_arguments);

    $form_state->setRedirect('entity.helpdesk_issue.canonical', [
      'helpdesk_issue' => $entity->id(),
    ]);

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
      '#submit' => ['::submitForm', '::save'],
    ];
    return $form;
  }

  /**
   * Sets the renderer.
   *
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer.
   */
  public function setRenderer(Renderer $renderer): void {
    $this->renderer = $renderer;
  }

}
